#!/bin/bash
#$ -S /bin/sh
#$ -N demo-cluster-job
#$ -cwd
source /etc/profile.d/modules.sh
module load modules modules-init modules-gs modules-noble
module load fftw gmp mpfr gcc
module load python/2.6.1
module load lmutil/10.8.5
module load matlab/2008b

laa=5
lss=0

#n_proteins=(100 250 500 1000 5000 15000 50000)
n_proteins=(100 250 500 1000 5000 15000 30000)
percentages=(0 5 10 15 20)
n_graphs=$((3**(lss+1)))
max_n_iterations_random=11

n_samples=${#n_proteins[@]}
n_percentages=${#percentages[@]}

path_root="lss_${lss}_synthetic_"
scripts_root="scripts/"
results_root="results/a_date/"
data_root="data/"
#output_path_1="/net/noble/vol2/home/zafer/projects/pssp/results/2009-10-16/"
#output_path_2="/net/noble/vol2/home/zafer/projects/pssp/results/"
#output_path_3="/~zafer/projects/pssp/results/2009-10-16/"
output_path_1="/net/noble/vol2/home/zafer/projects/pssp/results/sparse_models/synthetic_data/laa_5_lss_0_perc_20/"
output_path_2="/net/noble/vol2/home/zafer/projects/pssp/results/"
output_path_3="/~zafer/projects/pssp/results/sparse_models/synthetic_data/laa_5_lss_0_perc_20/"

output_root="progressive_EM"

fixed_params_root_1="fixed_params"
fixed_params_root_2="fixed_params.txt"
learned_params_root_1="learnedParams.EM.dlinkMats"

#matrix2png="/noble/bin/matrix2png"
matrix2png="/net/noble/vol2/home/zafer/software/matrix2png-1.0.7/matrix2png"
color_map_filename="my_color_map"
perc_to_clip_color_scale=0

percentages_filename="percentages"
n_proteins_filename="n_proteins"
iteration_indices_filename="iteration_indices"
n_aas_filename="number_of_amino_acids_$output_root"
matlab_commandlines_root="matlab_commandlines"
matlab_commandlines_filename_main="matlab_commandlines.m"
html_output="output_${output_root}.html"

rm -rf $percentages_filename
rm -rf $n_proteins_filename
rm -rf $iteration_indices_filename

# Compute the number of amino acids at each set that contains a given number of synthetic proteins
obs_root="synthetic"
n_aa_filename_root="n_aa_synthetic_"

output_filename_to_be_catted=""

for ((i=0; i < n_samples; i++))
do
   path_name_core=$path_root${n_proteins[$i]}/
   data_dir=${path_name_core}$data_root
   output_filename="$n_aa_filename_root${n_proteins[$i]}"

#   python compute_total_number_of_amino_acids.py $data_dir $obs_root ${n_proteins[$i]} $laa $output_filename
   output_filename_to_be_catted="$output_filename_to_be_catted $output_filename"
done
   
cat $output_filename_to_be_catted >& $n_aas_filename

# Compute the matrix2png plots that show the difference between the learned parameters from synthetic data and real data 
for ((iteration_random = 1; iteration_random < max_n_iterations_random; iteration_random++))
do

   rm -rf $n_proteins_filename

   for ((i=0; i < n_samples; i++))
   do
      path_name_core=$path_root${n_proteins[$i]}/
      scripts_dir=${path_name_core}$scripts_root
      results_dir=${path_name_core}$results_root
      data_dir=${path_name_core}$data_root

      for ((j=0; j < n_percentages; j++))
      do
         output_root_discrete=${output_root}_discrete_it_${iteration_random}_nproteins_${n_proteins[$i]}_perc_${percentages[$j]}
         output_root_continuous=${output_root}_continuous_it_${iteration_random}_nproteins_${n_proteins[$i]}_perc_${percentages[$j]}
         output_root_accuracies=${output_root}_accuracies_it_${iteration_random}_nproteins_${n_proteins[$i]}_perc_${percentages[$j]}
         output_root_minmax=${output_root}_minmax_it_${iteration_random}_nproteins_${n_proteins[$i]}_perc_${percentages[$j]}

        ./generate_dlink_difference_plots.sh ./ $laa $lss $data_dir${fixed_params_root_1}_${percentages[$j]}.txt $data_dir${learned_params_root_1}_${percentages[$j]}.out $results_dir${fixed_params_root_2}_${iteration_random}_${percentages[$j]} $results_dir${learned_params_root_1}_${iteration_random}_${percentages[$j]}.out $matrix2png $color_map_filename $perc_to_clip_color_scale ${output_path_1}$output_root_discrete ${output_path_1}$output_root_continuous ${output_path_1}$output_root_accuracies ${output_path_1}$output_root_minmax

      done

      echo ${n_proteins[$i]} >> $n_proteins_filename

   done
   
   echo $iteration_random >> $iteration_indices_filename

done

for ((j=0; j < n_percentages; j++))
do  
   echo ${percentages[$j]} >> $percentages_filename
done

#for ((iteration_random = 1; iteration_random < max_n_iterations_random; iteration_random++))
#do  

#   matlab_commandlines_filename=${matlab_commandlines_root}_it_$iteration_random
#   python generate_accuracy_files.py $n_proteins_filename $percentages_filename $n_graphs $output_path_1 $output_root $output_path_1$matlab_commandlines_filename
#   echo $output_path_1$matlab_commandlines_filename >> $output_path_1$matlab_commandlines_filename_main
#done

# Compute accuracy measures
output_root2=${output_root}_accuracies
matlab_commandlines_filename2=${matlab_commandlines_root}2.m  
python generate_accuracy_files2.py $iteration_indices_filename $n_proteins_filename $percentages_filename $n_graphs $output_path_1 $output_root2 $output_path_1$matlab_commandlines_filename2

\cp $iteration_indices_filename $output_path_1
\cp $n_proteins_filename $output_path_1
\cp $percentages_filename $output_path_1
\cp $n_aas_filename $output_path_1
 
# Generate an html file that reports all the results
python generate_html_file.py $n_proteins_filename $n_aas_filename $percentages_filename $n_graphs $output_path_1 $output_path_3 $output_root $html_output
 
