__author__ = "Zafer Aydin"
__date__ = "02/10/2009"

# FILENAME: generate_initial_means_covars_dlink_matrices2.py
# PROJECT: Protein Secondary Structure Prediction

usage="""

	Usage: generate_initial_means_covars_dlink_matrices2.py <number of Gaussian components> <number of dimensions> <init_covariance> <starting_index> <output filename> <pssm window size> <obs files list filename>
"""
 
import sys
import random
import math

if __name__ == "__main__":

	if ( len(sys.argv) != 8 ):

		sys.stderr.write(usage)
		sys.exit(-1)

	n_gc = int(sys.argv[1])
	n_dimensions = int(sys.argv[2])
	init_covariance = float(sys.argv[3])
	starting_index = int(sys.argv[4])
	output_filename = sys.argv[5]
	pssm_window_size = int(sys.argv[6])
	obs_list_filename = sys.argv[7]

	output_file = open(output_filename, 'w')

	output_file2 = open('means_covariances.txt', 'w')

	#Compute initial mean and diagonal covariance vectors from the data
	obs_list_file = open(obs_list_filename, 'r')
	mean_profile_vector_helix = [0.0]*n_dimensions
	mean_profile_vector_strand = [0.0]*n_dimensions
	mean_profile_vector_loop = [0.0]*n_dimensions

	total_number_of_vectors_helix = 0
	total_number_of_vectors_strand = 0
	total_number_of_vectors_loop = 0

	for obs_filename in obs_list_file:

		obs_filename = obs_filename.rstrip()
		obs_file = open(obs_filename, 'r')
		
		done = 0
		number_of_vectors_so_far=0

		while not done:

			if (number_of_vectors_so_far < pssm_window_size):

				number_of_vectors_so_far += 1
				continue

			profile_vector = obs_file.readline()

			if (len(profile_vector) == 0):

				done = 1
				continue

			profile_vector = profile_vector.rstrip()
			profile_vector_token = profile_vector.split()
		
			ss_label = int(profile_vector_token[n_dimensions])

			if (ss_label == 0):
	
				for i in range(n_dimensions):

					mean_profile_vector_helix[i] += float(profile_vector_token[i])			

				total_number_of_vectors_helix += 1

			elif (ss_label == 1):

				for i in range(n_dimensions):

                                        mean_profile_vector_strand[i] += float(profile_vector_token[i])

				total_number_of_vectors_strand += 1

			else:

				for i in range(n_dimensions):

                                        mean_profile_vector_loop[i] += float(profile_vector_token[i])

                                total_number_of_vectors_loop += 1


			number_of_vectors_so_far += 1

		obs_file.close()

	obs_list_file.close()

	for i in range(n_dimensions):

		if (total_number_of_vectors_helix != 0):
        		mean_profile_vector_helix[i] /= float(total_number_of_vectors_helix)
		else:
			 mean_profile_vector_helix[i] = 0.0

		if (total_number_of_vectors_strand != 0):
			mean_profile_vector_strand[i] /= float(total_number_of_vectors_strand)
		else:
			mean_profile_vector_strand[i] = 0.0

		if (total_number_of_vectors_loop != 0):
			mean_profile_vector_loop[i] /= float(total_number_of_vectors_loop)		 
		else:
			mean_profile_vector_loop[i] = 0.0

	#print 'mean profile vector helix = %s ' % mean_profile_vector_helix

	mean_diag_covariance_vector_helix = [0]*n_dimensions
	mean_diag_covariance_vector_strand = [0]*n_dimensions
	mean_diag_covariance_vector_loop = [0]*n_dimensions

	mean_covariance_vector_helix = []

	for i in range(n_dimensions):

		mean_covariance_vector_helix.append([])

		for j in range(n_dimensions):

			mean_covariance_vector_helix[i].append(0)
	
	mean_covariance_vector_strand = []

        for i in range(n_dimensions):

                mean_covariance_vector_strand.append([])

                for j in range(n_dimensions):

                        mean_covariance_vector_strand[i].append(0)

	mean_covariance_vector_loop = []

        for i in range(n_dimensions):

                mean_covariance_vector_loop.append([])

                for j in range(n_dimensions):

                        mean_covariance_vector_loop[i].append(0)

	obs_list_file = open(obs_list_filename, 'r')
        total_number_of_vectors_helix = 0
	total_number_of_vectors_strand = 0
	total_number_of_vectors_loop = 0

        for obs_filename in obs_list_file:

		obs_filename = obs_filename.rstrip()
                obs_file = open(obs_filename, 'r')

                done = 0
                number_of_vectors_so_far=0

                while not done:

                        if (number_of_vectors_so_far < pssm_window_size):
			
				number_of_vectors_so_far += 1
				continue

                        profile_vector = obs_file.readline()

			if (len(profile_vector) == 0):

                                done = 1
                                continue

                        profile_vector = profile_vector.rstrip()
                        profile_vector_token = profile_vector.split()

			ss_label = int(profile_vector_token[n_dimensions])
                        
			if (ss_label == 0):
                      
				difference_vector = [0]*n_dimensions
 
        	                for i in range(n_dimensions):

                	                difference_vector[i] = float(profile_vector_token[i]) - mean_profile_vector_helix[i]

				for i in range(n_dimensions):

					mean_diag_covariance_vector_helix[i] += difference_vector[i]*difference_vector[i]
	                       		number_of_vectors_so_far += 1

				for i in range(n_dimensions):

			                for j in range(n_dimensions):

						mean_covariance_vector_helix[i][j] += difference_vector[i]*difference_vector[j]

				total_number_of_vectors_helix += 1

			elif (ss_label == 1):

				difference_vector = [0]*n_dimensions

                                for i in range(n_dimensions):

                                        difference_vector[i] = float(profile_vector_token[i]) - mean_profile_vector_strand[i]

                                for i in range(n_dimensions):

                                        mean_diag_covariance_vector_strand[i] += difference_vector[i]*difference_vector[i]
                                        number_of_vectors_so_far += 1

                                for i in range(n_dimensions):

                                        for j in range(n_dimensions):

                                                mean_covariance_vector_strand[i][j] += difference_vector[i]*difference_vector[j]

				total_number_of_vectors_strand += 1

			else:

				difference_vector = [0]*n_dimensions

                                for i in range(n_dimensions):

                                        difference_vector[i] = float(profile_vector_token[i]) - mean_profile_vector_loop[i]

                                for i in range(n_dimensions):

                                        mean_diag_covariance_vector_loop[i] += difference_vector[i]*difference_vector[i]
                                        number_of_vectors_so_far += 1

                                for i in range(n_dimensions):

                                        for j in range(n_dimensions):

                                                mean_covariance_vector_loop[i][j] += difference_vector[i]*difference_vector[j]


	                        total_number_of_vectors_loop += 1

                obs_file.close()

        obs_list_file.close()
	
	for i in range(n_dimensions):

		if (total_number_of_vectors_helix != 0):
			mean_diag_covariance_vector_helix[i] /= float(total_number_of_vectors_helix)
		else:
			mean_diag_covariance_vector_helix[i] = 10.0

		if (total_number_of_vectors_strand != 0):
			mean_diag_covariance_vector_strand[i] /= float(total_number_of_vectors_strand)
		else:
			mean_diag_covariance_vector_strand[i] = 10.0

		if (total_number_of_vectors_loop != 0):
			mean_diag_covariance_vector_loop[i] /= float(total_number_of_vectors_loop)
		else:
			mean_diag_covariance_vector_loop[i] = 10.0

	for i in range(n_dimensions):

	        for j in range(n_dimensions):

        		mean_covariance_vector_helix[i][j] /= float(total_number_of_vectors_helix)
			mean_covariance_vector_strand[i][j] /= float(total_number_of_vectors_strand)
			mean_covariance_vector_loop[i][j] /= float(total_number_of_vectors_loop)

	#print 'mean covariance vector helix = %s ' % mean_covariance_vector_helix
	
	for i in range(n_dimensions):
	
		output_file2.write('%f ' % mean_profile_vector_helix[i] )

	output_file2.write('\n')
	
	for i in range(n_dimensions): 
        
                output_file2.write('%f ' % mean_profile_vector_strand[i] )

	output_file2.write('\n')

        for i in range(n_dimensions): 
        
                output_file2.write('%f ' % mean_profile_vector_loop[i] )

	output_file2.write('\n')
	output_file2.write('\n')

	for i in range(n_dimensions):

                for j in range(n_dimensions):

			output_file2.write('%.15f ' % mean_covariance_vector_helix[i][j] )
		
		output_file2.write('\n')

	output_file2.write('\n')

	for i in range(n_dimensions): 
 
                for j in range(n_dimensions):
 
                        output_file2.write('%.15f ' % mean_covariance_vector_strand[i][j] )
                                                
                output_file2.write('\n')
         
        output_file2.write('\n')
	
	for i in range(n_dimensions): 
 
                for j in range(n_dimensions):
 
                        output_file2.write('%.15f ' % mean_covariance_vector_loop[i][j] )
                                                
                output_file2.write('\n')
         
        output_file2.write('\n')

	#Generate initial mean vectors
	output_file.write('%d' %n_gc)
        output_file.write('\n\n')

        for i in range(n_gc):

                output_file.write('  ')
                output_file.write('%d' % i)
                output_file.write('   ')
                output_file.write('meanEC')
                output_file.write(str(i))
                output_file.write('    ')
                output_file.write('%d' % n_dimensions)

                for j in range(n_dimensions):

                        output_file.write('   ')
                        #a_random_value = random.normalvariate(0,0.2/3.0)
                        #a_random_value = random.uniform(0,1.0)
			a_random_value = 0.0 

			if (i == 0):

				initial_mean_value = mean_profile_vector_helix[j] + a_random_value*math.sqrt(mean_diag_covariance_vector_helix[j])

			elif (i == 1):

				initial_mean_value = mean_profile_vector_strand[j] + a_random_value*math.sqrt(mean_diag_covariance_vector_strand[j])

			else:
				initial_mean_value = mean_profile_vector_loop[j] + a_random_value*math.sqrt(mean_diag_covariance_vector_loop[j])

                        output_file.write('%.15f' % initial_mean_value)
			
                output_file.write('\n')

	output_file.write('\n')

	#Generate initial diagonal entries of covariance matrix
        output_file.write('%d' %n_gc)
        output_file.write('\n\n')

        for i in range(n_gc):

                output_file.write('  ')
                output_file.write('%d' % i)
                output_file.write('   ')
                output_file.write('covarEC')
                output_file.write(str(i))
                output_file.write('    ')
                output_file.write('%d' % n_dimensions)

                for j in range(n_dimensions):

                        output_file.write('   ')

			if (i == 0):

				initial_covariance_value = mean_diag_covariance_vector_helix[j]

			elif (i==1):

				initial_covariance_value = mean_diag_covariance_vector_strand[j]

			else:

				initial_covariance_value = mean_diag_covariance_vector_loop[j]

                        output_file.write('%.15f' % initial_covariance_value)

                output_file.write('\n')

	output_file.write('\n')

	#Generate initial dlink matrices
	output_file.write('%d' %n_gc)
	output_file.write('\n')

	for i in range(n_gc):

		output_file.write('%d' % i)
		output_file.write('\n')
		output_file.write('dlink_mat')
		output_file.write('%d' % i)
		output_file.write('\n')
		output_file.write('dlink_str' + str(i))
		output_file.write('\n')
		output_file.write('%d' % n_dimensions)
		output_file.write('\n')

		for j in range(n_dimensions):

			n_dependencies = starting_index - j
			output_file.write('%d' % n_dependencies)
			output_file.write('  ')

			for k in range(n_dependencies):

				output_file.write('%.1f' % 0.0)
				output_file.write(' ')

			output_file.write('\n')

		output_file.write('\n')
	
	output_file.close()
	output_file2.close() 
