__author__ = "Zafer Aydin"
__date__ = "01/23/2009"

# FILENAME: quadriplet_histogram.py 
# PROJECT: Protein Secondary Structure Prediction

usage = """

	python quadriplet_histogram.py <dataset_filename> <output_filename>
"""

def give_me_ss_index(ss_label):

	return ((ss_label == 'H')*0 + (ss_label == 'E')*1 + (ss_label == 'L')*2)

def give_me_quadriplet_index( ss_label_i, ss_label_i_1, ss_label_i_2, ss_label_i_3 ):

	ss_index_i = give_me_ss_index(ss_label_i)
	ss_index_i_1 = give_me_ss_index(ss_label_i_1)
	ss_index_i_2 = give_me_ss_index(ss_label_i_2)
	ss_index_i_3 = give_me_ss_index(ss_label_i_3)

	quadriplet_index = 27*ss_index_i_3 + 9*ss_index_i_2 + 3*ss_index_i_1 + ss_index_i
	return quadriplet_index

def give_me_quadriplet_from_index( index ):

	quad1 = index / 27
	rem1 = index % 27

	quad2 = rem1 / 9
	rem2 = rem1 % 9

	quad3 = rem2 / 3
	rem3 = rem2 % 3

	quadriplet = [0]*4
	quadriplet[0] = quad1+1
	quadriplet[1] = quad2+1
	quadriplet[2] = quad3+1
	quadriplet[3] = rem3+1

	return quadriplet

def reduce_from_8_to_3_state_EHL_mapping( ss_seq ):

        str_length = len(ss_seq)

        ss_seq_red = ''

        for i in range(0, str_length, 1):

                if ( (ss_seq[i] == 'H') or (ss_seq[i] == 'G') or (ss_seq[i] == 'I') ):

                        ss_seq_red += 'H'

                elif ( (ss_seq[i] == 'E') or (ss_seq[i] == 'B') ):

                        ss_seq_red += 'E'

                else:
                        ss_seq_red += 'L'

        return ss_seq_red

import sys

if __name__ == "__main__":


	if (len(sys.argv) != 3):

		sys.stderr.write(usage)
		sys.exit(-1)

	dataset_filename = sys.argv[1]
	output_filename = sys.argv[2]

	dataset_file = open( dataset_filename, 'r')
	output_file = open( output_filename, 'w')

	quadriplet_counts = [0.0]*81

	done = 0
	while not done:

		# Fasta id
		a_line = dataset_file.readline()
		
		if (len(a_line) == 0):

			done = 1
			continue
		
		# Amino acid and secondary structure sequence
		a_line = dataset_file.readline()
		aa_seq = a_line.rstrip()
		seq_length = len(aa_seq)

		a_line = dataset_file.readline()
		ss_seq = a_line[0:seq_length]
		
		ss_seq = reduce_from_8_to_3_state_EHL_mapping(ss_seq)

		for i in range(3, seq_length, 1):

			ss_label_i = ss_seq[i]
			ss_label_i_1 = ss_seq[i-1]
			ss_label_i_2 = ss_seq[i-2]
			ss_label_i_3 = ss_seq[i-3]
			quadriplet_index = give_me_quadriplet_index(ss_label_i, ss_label_i_1, ss_label_i_2, ss_label_i_3)
			quadriplet_counts[quadriplet_index] += 1.0
	
	sum_quadriplet = sum(quadriplet_counts)
	for i in range(len(quadriplet_counts)):

		quadriplet_counts[i] /= sum_quadriplet

	for i in range(len(quadriplet_counts)):

		quadriplet = give_me_quadriplet_from_index(i)
		output_file.write('%d' % quadriplet[0])
		output_file.write('%d' % quadriplet[1])
		output_file.write('%d' % quadriplet[2])
		output_file.write('%d' % quadriplet[3])
		output_file.write('\t')
		output_file.write('%f' % quadriplet_counts[i])
		output_file.write('\n')

	output_file.close()		
	dataset_file.close()
