function run_dbn_in_generative_mode(data_dir, laa, learned_means_filename, learned_diag_covars_filename, learned_dlink_mats_filename, learned_dense_cpts_filename, learned_dlink_structure_filename, iteration_count, n_proteins_in_training_set)
 
format long;

%data_dir='./';  
%learned_means_filename = 'learnedParams.EM.meanVecs.out';
%learned_diag_covars_filename = 'learnedParams.EM.covarVecs.out';
%learned_dlink_mats_filename = 'learnedParams.EM.dlinkMats.out';
%learned_dense_cpts_filename = 'learnedParams.EM.denseCPTs.out';
%learned_dlink_structure_filename = 'fixed_params.txt';
%n_proteins_in_training_set = 400;
%laa = 5;

n_dim = 20;
n_gc = 3;

%Read means
GMTK_means = read_mean_vectors(learned_means_filename, n_gc, n_dim);
GMTK_diag_covars = read_covar_vectors(learned_diag_covars_filename, n_gc, n_dim);
GMTK_dlinks = read_dlink_matrices(learned_dlink_mats_filename, learned_dlink_structure_filename, n_gc, n_dim, laa);
[start_length_cpt, length_cpt_first_dmax_pos, state_transition_cpt, exponential_tail_cpt] = read_dense_cpts(learned_dense_cpts_filename);

%Convert the GMTK's means, covariances and dlinks to regular mean and
%covariance representation for multinormals
[mean_shifts, U2s, covariances] = convert_gmtk_format_to_regular_means_covs(GMTK_means, GMTK_diag_covars, GMTK_dlinks, n_gc, n_dim, laa);
    
%Generate length cpt for the first M segment lengths
M=30;
n_ss_labels = 3;
Dmax = 13;
length_cpt = generate_length_cpt(length_cpt_first_dmax_pos, exponential_tail_cpt, M, n_ss_labels, Dmax);

n_proteins = n_proteins_in_training_set; %we are going to generate observation vectors for this many proteins
average_n_aa = 155;
synthetic_data_filename_root = 'synthetic';

%Xs=[];

for j=1:n_proteins
   
    %j
    %backup_filename = num2str(j);
    observation_history=[];
    backup_filename = data_dir;
    backup_filename = strcat(backup_filename, num2str(1));
    backup_filename = strcat(backup_filename, '.obs');
    fid_backup_positions = fopen(backup_filename, 'r');

    synthetic_data_filename = data_dir;
    synthetic_data_filename = strcat(synthetic_data_filename, synthetic_data_filename_root);
    %synthetic_data_filename = strcat(synthetic_data_filename, '_');
    %synthetic_data_filename = strcat(synthetic_data_filename, num2str(iteration_count));
    synthetic_data_filename = strcat(synthetic_data_filename, '_');
    synthetic_data_filename = strcat(synthetic_data_filename, num2str(j));
    synthetic_data_filename = strcat(synthetic_data_filename, '.obs');
        
    fid_synthetic = fopen(synthetic_data_filename, 'w');
    
    for k=1:laa
    
        backup_vector = [];
        a_line = fgetl(fid_backup_positions);
        [token,remainder]=strtok(a_line);
        backup_vector=[backup_vector; str2num(token)];
        
        for kk=1:19
            [token,remainder]=strtok(remainder);
            backup_vector=[backup_vector; str2num(token)];
        end
        
        observation_history = [backup_vector; observation_history];
        
        fprintf(fid_synthetic, '%s\n', a_line);
    end
    
    n_vectors_per_protein = 0;
    
    %Generate data for the first position
    position_index = 1;
    q = 3; %set the first position to loop state
    segment_length = generate_discrete_state(length_cpt, q);

    if (segment_length >= Dmax)
        
        n_positions_with_dmax = segment_length - Dmax + 1;
    else
        n_positions_with_dmax = 0;
    end
    
    state_count_down = give_me_state_count_down(segment_length, Dmax);
    change_state = give_me_change_state(state_count_down);
    state_label_history = q;
    
    %Generate a multinormal data sample
    x=generate_multinormal_data_sample(U2s, observation_history, mean_shifts, covariances, q);
    observation_history=[transpose(x);observation_history(1:((laa-1)*n_dim))];
    
    data_vector = [x, state_count_down, change_state];
    n_data = length(data_vector);
    
    for k=1:n_dim,
    
        fprintf(fid_synthetic, '%.7f ', data_vector(k));
    end
    
    fprintf(fid_synthetic, '%d ', (q-1));
    fprintf(fid_synthetic, '%d ', (state_count_down-1));
    fprintf(fid_synthetic, '%d\n', change_state);
    
    q_pre = q;
    change_state_pre = change_state;
    state_count_down_pre = state_count_down;
        
    if (n_positions_with_dmax > 0)
    
        n_positions_with_dmax = n_positions_with_dmax - 1;
    end
    
    done = 0;
    while (~done)
        
       if (change_state_pre == 0) 
          
            q=q_pre;
            
            if (n_positions_with_dmax == 0)
               
                state_count_down = state_count_down_pre - 1;
                change_state = give_me_change_state(state_count_down);
                
            else
                state_count_down = Dmax;
                change_state = give_me_change_state(state_count_down);
                n_positions_with_dmax = n_positions_with_dmax - 1;
            end
                        
       else  
            %Generate a new secondary structure segment
            q = generate_discrete_state(state_transition_cpt, q_pre);
            
            %Generate a new segment length       
            segment_length = generate_discrete_state(length_cpt, q);

            if (segment_length >= Dmax)

                n_positions_with_dmax = segment_length - Dmax + 1;
            else
                n_positions_with_dmax = 0;
            end

            state_count_down = give_me_state_count_down(segment_length, Dmax);
            change_state = give_me_change_state(state_count_down);
       end
       
       %Generate a multinormal data sample
       x=generate_multinormal_data_sample(U2s, observation_history, mean_shifts, covariances, q);
       observation_history=[transpose(x);observation_history(1:((laa-1)*n_dim))];
           
       %if (q == 1)
       %     Xs=[Xs;x];
       %end
       
       data_vector = [x, state_count_down, change_state];
       n_data = length(data_vector);

       for k=1:n_dim,

           fprintf(fid_synthetic, '%.7f ', data_vector(k));
       end

       fprintf(fid_synthetic, '%d ', (q-1));
       fprintf(fid_synthetic, '%d ', (state_count_down-1));
       fprintf(fid_synthetic, '%d\n', change_state);

       q_pre = q;
       change_state_pre = change_state;
       state_count_down_pre = state_count_down;
       position_index = position_index + 1;
       
       if ((position_index >= average_n_aa) && (state_count_down == 1))
       
           done = 1;
       end
    end
    
    fclose(fid_backup_positions);
    fclose(fid_synthetic);
end

