__author__ = "Zafer Aydin"
__date__ = "01/22/2009"

# FILE: convert_fasta_time_lag.py 
# PROJECT: Protein Secondary Structure Prediction

usage = """

USAGE: python convert_fasta_time_lag.py <number_of_state_classes> <data_dir> <obs_files_list> <postp_files_list_forward_model> <postp_files_list_reverse_model> <timelag> <true_ss_file> <predicted_ss_file>

DESCRIPTION: This script, takes the list of .obs files (<obs_files_list>) and the list of .postp files (<postp_files_list>) as input. .obs files contain the observation data and true secondary structure labels represented by numbers and .postp files contain the posterior probability values of each possible state for each amino acid of a protein. The script outputs true secondary structure assignments (<true_ss_file>) as well as the predicted secondary structure labels (<predicted_ss_file>).
"""

ss_alphabet_8 = "HBEGITSLLL"   #Label 9 and 10 are also L in SS labels
ss_alphabet_3 = "HEL"

def generate_true_ss_file(number_of_state_classes, data_dir, obs_files_list_filename, time_lag, true_ss_filename):

	obs_files_list_file = open( obs_files_list_filename, 'r' )
	true_ss_file = open( true_ss_filename, 'w' )

	for line in obs_files_list_file:

		obs_filename = line.rstrip()
		obs_file = open( obs_filename, 'r' )
		
		true_ss_labels_str = ''
		line_counter = 0

		for obs_line in obs_file:
			
			if (line_counter < time_lag):
				line_counter += 1
				continue
			
			token_list = obs_line.split()	
			#ss_label_int = int( token_list[-1] )
			ss_label_int = int( token_list[-3] )

			#print ss_label_int
			
			if (number_of_state_classes == 3):

				true_ss_labels_str += ss_alphabet_3[ss_label_int]  
			
			elif (number_of_state_classes == 8):
			
				true_ss_labels_str += ss_alphabet_8[ss_label_int]
	
		obs_file.close()

		protein_id = obs_filename.replace(data_dir, '')
		protein_id = protein_id.replace('.obs', '')

		true_ss_file.write( '>' + protein_id + '\n')
		true_ss_file.write(true_ss_labels_str + '\n');

	true_ss_file.close()
	obs_files_list_file.close()


def generate_predicted_ss_file( number_of_state_classes, data_dir, postp_files_list_filename_forward, postp_files_list_filename_reverse, pred_ss_filename ):

	postp_files_list_file_forward = open( postp_files_list_filename_forward, 'r' )
	postp_files_list_file_reverse = open( postp_files_list_filename_reverse, 'r' )

	pred_ss_file = open( pred_ss_filename, 'w' )

	done  = 0

	#print postp_files_list_filename_forward
	#print postp_files_list_filename_reverse

	while not done:
		
		line_forward = postp_files_list_file_forward.readline()
		line_reverse = postp_files_list_file_reverse.readline()
		
		if (( len( line_forward ) == 0 ) or ( len( line_reverse ) == 0 )):
		#if ( len( line_forward ) == 0 ):

			done = 1
			continue

		postp_filename_forward = line_forward.rstrip()
		postp_filename_reverse = line_reverse.rstrip()

		postp_file_forward = open( postp_filename_forward, 'r' )
		postp_file_reverse = open( postp_filename_reverse, 'r' )

		pred_ss_labels_str = ''

		line_counter = 1

		postp_file_forward_inside =  postp_file_forward.readlines()
                postp_file_reverse_inside =  postp_file_reverse.readlines()

		postp_file_forward_inside_length = len(postp_file_forward_inside)
		postp_file_reverse_inside_length = len(postp_file_reverse_inside)

		if (postp_file_forward_inside_length != postp_file_reverse_inside_length):
			
			print postp_file_forward_inside_length
			print postp_file_reverse_inside_length
			print 'Error the length of the postp files for the forward and reverse models are not the same'
			sys.exit(-1)

		for i in range(0, postp_file_forward_inside_length, 1):
		
			postp_line_forward = postp_file_forward_inside[i]
			postp_line_reverse = postp_file_reverse_inside[-1-i]

			token_list_forward = postp_line_forward.split()
			token_list_reverse = postp_line_reverse.split()
			
			#print '%s' % token_list_forward
			#print '%s' % token_list_reverse

			if ( len( token_list_forward ) == number_of_state_classes ):
				
				max_value = -1000
				max_index = 0			
	
				for i in range(0, len( token_list_forward ), 1):
				
					#Postp value is the average of postp values from the forward and reverse models
					postp_value = ( float( token_list_forward[i] ) + float( token_list_reverse[i] ) ) / 2.0
					#postp_value = float(token_list_forward[i])
					#postp_value = float(token_list_reverse[i])

					#print '%f' % float(token_list_forward[i])
					#print '%f' % float(token_list_reverse[i])
					#print '%f' % postp_value

					if ( postp_value > max_value):
						
						max_value = postp_value
						max_index = i
				
				#print '%s' % ssalphabet[max_index]

				if (number_of_state_classes == 3):
			 	
					pred_ss_labels_str += ss_alphabet_3[max_index]		
			
				elif (number_of_state_classes == 8):

					pred_ss_labels_str += ss_alphabet_8[max_index]
			
				#print max_index	
				#print pred_ss_labels_str

			line_counter += 1

		postp_file_forward.close()
		postp_file_reverse.close()

		#print '%s' % pred_ss_labels_str

		protein_id = postp_filename_forward.replace(data_dir, '')
		protein_id = protein_id.replace('.w_test.postp.sigmoid', '')
		protein_id = protein_id.replace('.test.postp.sigmoid', '')
		protein_id = protein_id.replace('.test.postp.linear', '')
		protein_id = protein_id.replace('.test.postp', '')

                pred_ss_file.write( '>' + protein_id + '\n' )
                pred_ss_file.write( pred_ss_labels_str + '\n' );

	pred_ss_file.close()
	postp_files_list_file_forward.close()
	postp_files_list_file_reverse.close()



#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# main program is here

import sys
import math

if __name__ == "__main__":
    
	if ( len(sys.argv) != 9 ):
		
		sys.stderr.write(usage)
		sys.exit(-1)
	
	number_of_state_classes = int(sys.argv[1])
	data_dir = sys.argv[2]
	obs_files_list_filename = sys.argv[3]
	postp_files_list_filename_forward = sys.argv[4]
	postp_files_list_filename_reverse = sys.argv[5]
	time_lag = int(sys.argv[6])
	true_ss_filename = sys.argv[7]
	pred_ss_filename = sys.argv[8]

	generate_true_ss_file( number_of_state_classes, data_dir, obs_files_list_filename, time_lag, true_ss_filename )
	generate_predicted_ss_file( number_of_state_classes, data_dir, postp_files_list_filename_forward, postp_files_list_filename_reverse, pred_ss_filename ) 


 


